# Generated by Django 4.0 on 2022-01-28 23:27

from django.db import migrations, models
import django.db.models.deletion
import hub20.apps.blockchain.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('blockchain', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ColdWallet',
            fields=[
                ('baseethereumaccount_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='blockchain.baseethereumaccount')),
            ],
            bases=('blockchain.baseethereumaccount',),
        ),
        migrations.CreateModel(
            name='HierarchicalDeterministicWallet',
            fields=[
                ('baseethereumaccount_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='blockchain.baseethereumaccount')),
                ('index', models.PositiveIntegerField(unique=True)),
            ],
            bases=('blockchain.baseethereumaccount',),
        ),
        migrations.CreateModel(
            name='KeystoreAccount',
            fields=[
                ('baseethereumaccount_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='blockchain.baseethereumaccount')),
                ('private_key', hub20.apps.blockchain.fields.HexField(max_length=64, unique=True)),
            ],
            bases=('blockchain.baseethereumaccount',),
        ),
    ]
